#!/usr/bin/python3
from ..generics import (core, file)
from . import prepper as mprepper
from . import processor as mprocessor

"""
Site Assembler: Context
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


class Context:

    def __init__(self, parent_level: int, source: str = ''):
        self.context_level = parent_level + 1
        self.source_tag_state = list(source.split('.'))
        self.source_tag = str(self.source_tag_state.pop(0))
        self.dataset = {}
        self.prepped_template = []
        self.result = ''

    # Data

    def get_dataset(self):
        return self.dataset

    def set_dataset(self, new_dataset):
        self.dataset.update(new_dataset)

    # Template

    def get_template(self):
        return self.prepped_template

    def set_template(self, raw_template):
        prepper = mprepper.Prepper()
        self.prepped_template.extend(prepper.run(raw_template,
                                                 self.source_tag,
                                                 self.source_tag_state))
        self.template_to_file()

    def template_to_file(self):
        template_report = []
        for action in self.get_template():
            template_report.append(
                f'''TAG: {action.get_action()}\nARGUMENTS:
                {action.get_argument()}\nCONTENT:\n{action.get_content()}
                \n-------------------------------------------------------
                ----------------------------------\n\n''')
        file.write_file(template_report, f'prepped-{self.context_level}')

    # Process

    def process(self):
        processor = mprocessor.Processor(self.context_level)
        self.result = processor.run(self.prepped_template, self.dataset)
        self.prepped_template = []
        return self.result



#!/usr/bin/python3
from ..generics import core
from . import tag as mtag
from . import token as mtoken
import re
from pyparsing import (Combine, printables, ZeroOrMore, MatchFirst,
                       quotedString, Word, CaselessKeyword, delimitedList)


"""
Site Assembler: Prepper
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


class Prepper:

    def __init__(self):
        self.prepped_template = [('ignore', '', '')]
        # tag, argument, full tag, start postition, end position
        self.tag_list = [('', '', '', 0, 0)]
        self.matching_tags = mtag.block_tags()
        self.multiblock_tags = mtag.multiblock_tags()
        self.tag_patterns = {
            'all': r'((?s).*)',
            'tags_list':
            r'({#[\s\S]*?[\s\S]#})|({{[\s\S]*?[\s\S]}})|({%[\s\S]*?[\s\S]%})',
        }

        self.match_literal = quotedString
        self.match_name = Word(printables, excludeChars='|:')
        self.match_argument = ZeroOrMore(
            ":" + MatchFirst(self.match_literal | self.match_name))
        self.match_filter = ZeroOrMore(
            "|" + self.match_name + self.match_argument)

        self.match_lit_w_argument = self.match_literal + self.match_argument
        self.match_name_w_argument = self.match_name + self.match_argument
        self.match_literal_w_filter = Combine(
            self.match_lit_w_argument + self.match_filter)
        self.match_name_w_filter = Combine(
            self.match_name_w_argument + self.match_filter)

        self.argument_patterns = {
            'all': ZeroOrMore(self.match_literal_w_filter
                              | self.match_name_w_filter),
            # 'if': ZeroOrMore(self.match_literal),
            'filter': ZeroOrMore(self.match_name_w_filter),
            'firstof': ZeroOrMore(self.match_literal | self.match_name),
            'for': ZeroOrMore(Combine(self.match_name_w_argument)
                              + CaselessKeyword("in")
                              + self.match_name_w_argument),
            # 'replace': MatchFirst(self.match_name_w_filter),
            'usedataset': MatchFirst(self.match_literal
                                     + ZeroOrMore(CaselessKeyword("as")
                                                  + self.match_name_w_filter)),
        }

    def is_block_tag(self, action_tag):
        return action_tag in self.matching_tags.keys()

    def is_end_tag(self, action_tag):
        return action_tag in self.matching_tags.values()

    def is_multiblock_tag(self, action_tag):
        return action_tag in self.multiblock_tags.keys()
    
    def is_multiblock_match(self, action_tag: str, main_multiblock_tag: str):
        if not core.is_empty(main_multiblock_tag):
            return action_tag in self.multiblock_tags.get(main_multiblock_tag)
        return False

    def is_multiblock_inner_tag(self, action_tag: str = '',
                                main_multiblock_tag: str = ''):
        if not core.is_empty(main_multiblock_tag):
            return self.is_multiblock_match(action_tag, main_multiblock_tag)
        else:
            for multiblock_tag in self.multiblock_tags.values():
                if action_tag in multiblock_tag:
                    return True
        return False
    



    def get_end_tag(self, action_tag):
        return self.matching_tags[action_tag]

    def pattern(self, pattern):
        if pattern not in self.tag_patterns.keys():
            pattern = 'all'
        return self.tag_patterns[pattern]

    def arguments(self, action_tag, argument_string):
        # Tags with Verbatim arguments
        verbatim = action_tag in (None,)  # Add tag for verbatim tags
        expression = action_tag in mtag.expression_argument_tags()
        if action_tag not in self.argument_patterns.keys():
            action_tag = 'all'

        if verbatim:
            token_list = [argument_string]
        else:
            match = self.argument_patterns[action_tag]
            token_list = delimitedList(match, ' ').parseString(
                argument_string).asList()

        tokens = []
        for token_string in token_list:
            token = mtoken.Token()
            token.build(token_string, verbatim, expression)
            tokens.append(token)
        return tuple(tokens)

    def tags_as_list(self, raw_template: str = ''):
        tag_list = []
        action_tag, argument, full_tag = ('',)*3
        for match in re.finditer(self.pattern('tags_list'), raw_template):
            if match.group(1):
                full_tag = str(match.group(1))
                action_tag = 'ignore'
                argument = ''
            elif match.group(2):
                full_tag = str(match.group(2))
                action_tag = 'replace'
                argument = ' '.join(full_tag.split()[1:-1])
            elif match.group(3):
                full_tag = str(match.group(3))
                action_tag = full_tag.split()[1].lower()
                argument = ' '.join(full_tag.split()[2:-1])
            tag_list.append((action_tag, argument, full_tag,
                             match.start(), match.end()))
        return tag_list

    def current_level_tags(self, tag_list: list = [], multiblock: str = ''):
        checked_tag_list = []
        running_level = 0
        end_tag = ''
        if core.not_empty(multiblock):
            running_level = + 1
        for action_tag, argument, full_tag, start, end in tag_list:
            if self.is_block_tag(action_tag):
                if running_level == 0:
                    checked_tag_list.append(
                        (action_tag, argument, full_tag, start, end))
                    end_tag = self.get_end_tag(action_tag)
                running_level += 1
            elif self.is_end_tag(action_tag):
                if running_level == 1 and action_tag == end_tag:
                    checked_tag_list.append(
                        (action_tag, argument, full_tag, start, end))
                    end_tag = ''
                running_level -= 1
            elif self.is_multiblock_inner_tag(action_tag, multiblock):
                if running_level == 1 \
                        and self.is_multiblock_match(action_tag, multiblock):
                    checked_tag_list.append(
                        (action_tag, argument, full_tag, start, end))
            else:
                if running_level == 0:
                    checked_tag_list.append(
                        (action_tag, argument, full_tag, start, end))
        return checked_tag_list

    def segments(self, tag_list: list, raw_template: str,
                 multiblock: str = ''):
        sliced = []
        list_index = 0

        # Copy content to single segment if no tags are present
        if len(tag_list) == list_index:
            sliced.append(mtag.Action('copy', (), raw_template))

        # Change tags into segments for processing
        while len(tag_list) > list_index:
            action_tag, argument, full_tag, tag_start, tag_end \
                = tag_list[list_index]
            argument = self.arguments(action_tag, argument)

            if len(tag_list) - list_index == 1:
                next_tag_start = len(raw_template)
            else:
                next_tag_start = tag_list[list_index + 1][3]

            # Copy segment before first tag encountered
            if list_index == 0:
                sliced.append(mtag.Action(
                    'copy', (), raw_template[0:tag_start]))

            if self.is_block_tag(action_tag):
                content = raw_template[tag_end:next_tag_start]
                sliced.append(mtag.Action(action_tag, argument, content))
            elif self.is_end_tag(action_tag):
                content = raw_template[tag_end:next_tag_start]
                sliced.append(mtag.Action('copy', (), content))
            elif self.is_multiblock_inner_tag(action_tag, multiblock):
                content = raw_template[tag_end:next_tag_start]
                sliced.append(mtag.Action(action_tag, argument, content))
            else:
                sliced.append(mtag.Action(action_tag, argument, ''))
                content = raw_template[tag_end:next_tag_start]
                sliced.append(mtag.Action('copy', (), content))

            list_index += 1
        return sliced

    def run(self, raw_template: str, src: str = '', src_state: list = []):
        # print(tags_as_list(raw_template))
        self.tag_list = self.current_level_tags(
            self.tags_as_list(raw_template))
        self.prepped_template = self.segments(self.tag_list, raw_template)
        return self.prepped_template



#!/usr/bin/python3
from ..generics import (core, file)
from . import context as mcontext

"""
Site Assembler: Template
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


class Template:

    def __init__(self, config: dict = {}):
        self.config = config
        self.processed_template = ''
        self.page_context = mcontext.Context(0)
        # Template
        self.page_template = file.file_to_str(
            self.config.get('sa_page_template'))
        self.page_context.set_template(self.page_template)
        # Dataset
        self.resolve_dataset(self.config.get('sa_page_dataset'))

    def set_dataset(self, new_dataset: dict):
        self.page_context.set_dataset(new_dataset)

    def resolve_dataset(self, dataset: list or str or dict):
        if not isinstance(dataset, list):
            dataset = [dataset]
        for src in dataset:
            if isinstance(src, str):
                # Future resolve id
                self.set_dataset(file.json_to_dict(src))
            elif isinstance(src, dict):
                self.set_dataset(src)

    def write_page_to_file(self):
        if 'sa_output_job' in self.config.keys():
            path_output_base = self.config.get('sa_output_job')
        else:
            path_output_base = ''
        if 'sa_output_site' in self.config.keys():
            path_output_site = self.config.get('sa_output_site')
        else:
            path_output_site = ''
        full_path = file.path_join(path_output_base, path_output_site)
        if 'sa_output_filename' in self.config.keys():
            path_output_filename = self.config.get('sa_output_filename')
        else:
            path_output_filename = ''
        file.write_file(self.processed_template,
                        path_output_filename, full_path)

    def process(self):
        self.processed_template = self.page_context.process()
        self.write_page_to_file()



#!/usr/bin/env python3
from pyparsing import (Combine, printables, ZeroOrMore, MatchFirst, Word,
                       quotedString, Group, delimitedList)

"""
Module Docstring
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


class Token:

    def __init__(self):
        self.token_string = ''
        # Token
        self.value = ''
        self.is_literal_ = False
        self.is_name_ = False
        self.is_verbatim_ = False
        self.is_expression_ = False
        # Argument
        self.has_argument_ = False
        self.argument_value = ''
        self.argument_is_literal = False
        self.argument_is_name = False
        # Filter
        self.is_filtered_ = False
        self.filter_list = []
        self.filter_value = ''
        self.filter_argument_value = ''
        self.filter_argument_is_literal = False
        self.filter_argument_is_name = False

    def __str__(self):
        return str(self.__class__) + ": " + str(self.__dict__)

    def __repr__(self):
        return f'{self.__class__} {self.token_string}'

    # Utilties
    def is_quoted(self, val: str):
        return val[0] == val[-1] and val[0] in ('\'', '\"')

    def unquote(self, val: str):
        return val[1:-1]

    # Token building
    def build(self, token_string: str, verbatim: bool = False,
              expression: bool = False):
        self.token_string = token_string
        self.is_verbatim_ = verbatim
        self.is_expression_ = expression

        match_literal = quotedString
        match_name = Word(printables, excludeChars='|:')
        match_argument = ZeroOrMore(
            Combine(":" + MatchFirst(match_literal | match_name)))
        match_filter = Group(ZeroOrMore(
            Group(Combine("|" + match_name) + match_argument)))
        match = (Group(match_literal + match_argument) + match_filter) \
            | (Group(match_name + match_argument) + match_filter)

        if self.is_verbatim_:
            match_list = [[token_string]]
        else:
            match_list = delimitedList(
                match, ' ').parseString(token_string).asList()

        # Set token
        if len(match_list) > 0:
            token_list = match_list.pop(0)
            if len(token_list) > 0:
                self.value = token_list.pop(0)
                if self.is_quoted(self.value):
                    if not self.is_verbatim_:
                        self.is_literal_ = True
                    self.value = self.unquote(self.value)
                else:
                    if not self.is_verbatim_:
                        self.is_name_ = True
            # Set argument
            if len(token_list) > 0:
                self.has_argument_ = True
                self.argument_value = token_list.pop(0)[1:]
                if self.is_quoted(self.argument_value):
                    self.argument_is_literal = True
                    self.argument_value = self.unquote(self.argument_value)
                else:
                    self.argument_is_name = True
        # Set filter
        if len(match_list) > 0:
            filter_list = match_list.pop(0)
            for filter in filter_list:
                self.is_filtered_ = True
                token_filter = filter.pop(0)[1:]
                f_arg = ''
                f_arg_type = ''
                if filter:
                    f_arg = filter.pop(0)[1:]
                    if self.is_quoted(f_arg):
                        f_arg = self.unquote(f_arg)
                        f_arg_type = 'literal'
                    else:
                        f_arg_type = 'name'
                self.filter_list.append((token_filter, f_arg, f_arg_type))

    def rebuild(self):
        self.build(self.token_string, self.is_verbatim_)

    # Token

    def get_value(self):
        return self.value

    def is_literal(self):
        return self.is_literal_

    def is_name(self):
        return self.is_name_

    def is_verbatim(self):
        return self.is_verbatim_

    def is_expression(self):
        return self.is_expression_

    # Argument

    def get_argument(self):
        return self.argument_value

    def is_argument_literal(self):
        return self.argument_is_literal

    def is_argument_name(self):
        return self.argument_is_name

    # Filter

    def is_filtered(self):
        return self.is_filtered_

    def has_next_filter(self):
        return len(self.filter_list) > 0

    def load_next_filter(self):
        if self.has_next_filter():
            f_value, f_arg_value, f_arg_type = self.filter_list.pop(0)
            self.filter_value = f_value
            self.filter_argument_value = f_arg_value
            if 'literal' in f_arg_type:
                self.filter_argument_is_literal = True
            elif 'name' in f_arg_type:
                self.filter_argument_is_name = True

    def get_filter_value(self):
        return self.filter_value

    def get_filter_argument(self):
        return self.filter_argument_value

    def is_filter_argument_literal(self):
        return self.filter_argument_is_literal

    def is_filter_argument_name(self):
        return self.filter_argument_is_name



#!/usr/bin/env python3
from . import prepper as mprepper

"""
Module Docstring
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


def valid_tags():
    pass


def block_tags():
    return {
        'comment': 'endcomment',
        'filter': 'endfilter',
        'for': 'endfor',
        'if': 'endif',
        'with': 'endwith',
    }


def multiblock_tags():
    return {
        'if': ['elif', 'else'],
    }


def expression_argument_tags():
    return ['if', 'elif', 'else']


class Action:

    def __init__(self, action: str = 'ignore', argument: tuple = (),
                 content: str = ''):
        self.main_action = action
        self.action = action
        self.argument = argument
        self.content = content
        # Multiblock
        self.is_multiblock_ = self.action in multiblock_tags().keys()
        self.multiblock_list = []
        if self.is_multiblock():
            self.build_multiblock()

    def __str__(self):
        return str(self.__class__) + ": " + str(self.__dict__)

    def __repr__(self):
        return f'{self.__class__} {self.action}'

    def get_action(self):
        return self.action

    def get_argument(self):
        return self.argument

    def get_content(self):
        return self.content

    def get(self):
        return (self.argument, self.content)

    def get_all(self):
        return (self.action, self.argument, self.content)

    # Multiblock
    def is_multiblock(self):
        return self.is_multiblock_

    def build_multiblock(self):
        mb_prep = mprepper.Prepper()
        tag_list = mb_prep.tags_as_list(self.get_content())
        tag_list = mb_prep.current_level_tags(tag_list, self.get_action())
        self.multiblock_list = mb_prep.segments(
            tag_list, self.get_content(), self.get_action())
        if self.multiblock_list[0].get_action() == 'copy':
            leading_action = self.multiblock_list.pop(0)
            self.content = leading_action.get_content()

    def has_next(self):
        return len(self.multiblock_list) > 0

    def next(self):
        if self.has_next():
            self.action, self.argument, self.content \
                    = self.multiblock_list.pop(0).get_all()



#!/usr/bin/python3
from ..generics import (core, file)
from . import context as mcontext
from . import tag as mtag
from . import token as mtoken
from . import token_filter as tf

"""
Site Assembler: Processor
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


class Processor:

    def __init__(self, context_level: int):
        self.context_level = context_level
        self.processed_template = []
        self.dataset = {}
        self.dataset_keyset = {}
        self.tagselect = {
            'comment': self.tag_comment,
            'copy': self.tag_copy,
            'filter': self.tag_filter,
            'firstof': self.tag_firstof,
            'for': self.tag_for,
            'if': self.tag_if,
            'ignore': self.tag_ignore,
            'length': self.tag_length,
            'replace': self.tag_replace,
            'usedataset': self.tag_usedataset,
            'usetemplate': self.tag_usetemplate,
        }

    def generate_dot_keys(self, obj: dict or list, current_key: str = ''):
        valid_set = set()
        if isinstance(obj, list):
            object_items = enumerate(obj)
        else:
            object_items = obj.items()
        for key, value in object_items:
            key = str(key)
            # Array
            if isinstance(value, list):
                if core.not_empty(current_key) \
                        and not current_key.endswith('.'):
                    current_key += '.'
                valid_set.add(current_key + key)
                item_index = 0
                for item in value:
                    valid_set.add(f'{current_key + key}.{item_index}')
                    if isinstance(item, (dict, list)):
                        valid_set.update(self.generate_dot_keys(
                            item, current_key + key))
                    item_index += 1
            # Dictionary
            elif isinstance(value, dict):
                if core.not_empty(current_key) \
                        and not current_key.endswith('.'):
                    current_key += '.'
                valid_set.add(current_key + key)
                valid_set.update(self.generate_dot_keys(
                    value, current_key + key))
            # Value
            else:
                if core.not_empty(current_key) \
                        and not current_key.endswith('.'):
                    current_key += '.'
                valid_set.add(current_key + key)
        return valid_set
        # [compact_list.append(x) for x in valid_list if x not in compact_list]

    def key_in_dataset(self, key: str, dataset: dict = None):
        if core.not_empty(key):
            if dataset is None:
                return key in self.dataset_keyset
            else:
                return key in self.generate_dot_keys(dataset)

    def get_data(self, return_type: str = 'copy', key: str = '',
                 dataset: dict = None):
        if dataset is None:
            return_value = self.dataset.copy()
        else:
            return_value = dataset.copy()
        if return_type.lower() == 'copy':
            return return_value
        if self.key_in_dataset(key) \
                and return_type in ('type', 'dict', 'list', 'str'):
            key = key.split('.')
            for step in key:
                if isinstance(return_value, dict) \
                        and step in return_value.keys():
                    return_value = return_value.get(step)
                elif isinstance(return_value, list):
                    try:
                        step = int(step)
                        if step in [x for x in range(len(return_value))]:
                            return_value = return_value[step]
                        else:
                            return_value = None
                    except:
                        break
                else:
                    return_value = None
                    break
            if return_type.lower() == 'type':
                if isinstance(return_value, dict):
                    return 'dict'
                elif isinstance(return_value, list):
                    return 'list'
            if return_type.lower() == 'dict':
                if isinstance(return_value, dict):
                    return return_value
                else:
                    # print(f'dict expected\nKey:
                    # {keys}\nValue: {return_value}')
                    return {}
            elif return_type.lower() == 'list':
                if isinstance(return_value, list):
                    return return_value
                else:
                    # print(f'dict expected\nKey:
                    # {keys}\nValue: {return_value}')
                    return []
            elif return_type.lower() in ('str'):
                if isinstance(return_value, (str, int)):
                    return str(return_value)
                else:
                    # print(f'str expected\nKey:
                    # {keys}\nValue: {return_value}')
                    return ''

    def update_dataset(self, newdata: dict):
        if core.not_empty(newdata):
            self.dataset.update(newdata)
            self.dataset_keyset = \
                self.generate_dot_keys(self.get_data('copy'))

    def apply_filter(self, token_value: str, filter_value: str,
                     filter_argument: str):
        filtered_value = token_value
        filter = filter_value
        argument = filter_argument
        if filter in tf.filterselect.keys():
            selected_filter = tf.filterselect[filter]
            filtered_value = selected_filter(filtered_value, argument)
        return filtered_value

    def resolve_filter(self, token, resolved_token):
        filtered_token = resolved_token
        if token.is_filtered():
            while token.has_next_filter():
                token.load_next_filter()
                filter_value = token.get_filter_value()
                filter_argument = token.get_filter_argument()
                if token.is_filter_argument_literal():
                    filter_argument = token.get_filter_argument()
                elif token.is_filter_argument_name():
                    filter_argument \
                        = self.get_data('str', token.get_filter_argument())
                filtered_token = self.apply_filter(resolved_token,
                                                   filter_value,
                                                   filter_argument)
        return filtered_token

    def resolve_token(self, token, alternative_value: str = None):
        resolved_token = ''
        token_value = token.get_value()
        key_in_dataset = self.key_in_dataset(token_value)
        if token.is_literal() or token.is_verbatim():
            resolved_token = token_value
        elif token.is_name():
            if key_in_dataset:
                resolved_token = self.get_data('str', token_value)
            elif token.is_expression():
                if token_value.count('.') > 0:
                    try:
                        resolved_token = str(float(token_value))
                    except:
                        resolved_token = 'None'
                else:
                    resolved_token = token_value
        resolved_token = self.resolve_filter(token, resolved_token)
        if token.is_expression():
            if token.is_literal() or (token.is_name() and key_in_dataset):
                resolved_token = '"' + resolved_token + '"'
        return resolved_token

    def evaluate(self, expression: tuple or str, eval_dataset: dict):
        unpacked_expression, evaluated = (None,)*2
        if type(expression) is tuple:
            if type(expression[0]) is mtoken.Token:
                expression = tuple(self.resolve_token(token)
                                   for token in expression)
            unpacked_expression = ' '.join(expression)
        elif type(expression) is str:
            unpacked_expression = expression
        else:
            unpacked_expression = str(expression)
        if core.not_empty(unpacked_expression):
            # Future: Alternative to eval()
            evaluated = eval(unpacked_expression, eval_dataset)
            if isinstance(evaluated, str):
                evaluated = core.is_none_or_empty(evaluated)
        return evaluated

    def new_context(self, template_segment: str, add_dataset: dict = {},
                    source: str = ''):
        newcontext = mcontext.Context(self.context_level, source)
        newcontext.set_dataset(self.get_data('copy'))
        newcontext.set_dataset(add_dataset)
        newcontext.set_template(template_segment)
        context_result = newcontext.process()
        del newcontext
        return context_result

    def process_action(self, action: mtag.Action):
        if action.get_action() in self.tagselect.keys():
            selected_tag = self.tagselect[action.get_action()]
        else:
            selected_tag = self.tag_ignore
        return selected_tag(action)

    def tag_comment(self, action: mtag.Action):
        return ''

    def tag_copy(self, action: mtag.Action):
        return action.get_content()

    def tag_filter(self, action: mtag.Action):
        filtered_content = self.new_context(
            action.get_content(), source='filter')
        for token in action.get_argument():
            if token.is_name():
                filter_value = token.get_value()
                filter_argument = token.get_argument()
                if token.is_argument_name():
                    filter_argument = self.get_data('str', filter_argument)
                filtered_content = self.apply_filter(filtered_content,
                                                     filter_value,
                                                     filter_argument)
                filtered_content = self.resolve_filter(token, filtered_content)
        return filtered_content

    def tag_firstof(self, action: mtag.Action):
        first_result = ''
        for argument in action.get_argument():
            first_result = self.resolve_token(argument)
            if first_result not in ('', None):
                return first_result
        return first_result

    def tag_for(self, action: mtag.Action):
        for_result = ''
        loop_key, assigner, dataset_name = (None,)*3
        arguments, content = action.get()
        if len(arguments) == 3:
            loop_key = arguments[0].get_value()
            assigner = arguments[1].get_value()
            dataset_name = arguments[2].get_value()
        if assigner == 'in' and dataset_name:
            for_dataset = self.get_data('list', dataset_name)
            for item in for_dataset:
                if isinstance(item, dict):
                    for_result += self.new_context(content,
                                                   {loop_key: item},
                                                   source='for')
        return for_result

    def tag_if(self, action: mtag.Action):
        def if_eval():
            ev = self.evaluate(argument, self.get_data('copy'))
            if not isinstance(ev, bool):
                ev = False
            return ev

        if_result = ''
        argument, content = action.get()
        evaluated = if_eval()
        if evaluated:
            if_result += self.new_context(content)
        elif not evaluated and action.has_next():
            while action.has_next():
                action.next()
                multiblock_action, argument, content = action.get_all()
                if multiblock_action == 'elif':
                    evaluated = if_eval()
                    if evaluated:
                        if_result += self.new_context(content)
                        break
                elif multiblock_action == 'else':
                    if_result += self.new_context(content)
                    break
        return if_result

    def tag_ignore(self, action: mtag.Action):
        return ''

    def tag_length(self, action: mtag.Action):
        length = ''
        arguments = action.get_argument()
        first_token = arguments[0]
        length = str(len(self.resolve_token(first_token)))
        if length == '0':
            data_type = self.get_data('type', first_token.get_value())
            if data_type in ('dict', 'list'):
                length = str(
                    len(self.get_data(data_type, first_token.get_value())))
        return length

    def tag_replace(self, action: mtag.Action):
        replace_value = self.resolve_token(action.get_argument()[0])
        return replace_value

    def tag_usedataset(self, action: mtag.Action):
        filename = ''
        dataset_content = {}
        source_token = action.get_argument()[0]
        if source_token.is_literal():
            filename = source_token.get_value()
        elif source_token.is_name():
            filename = self.get_data('str', source_token.get_value())
        if core.not_empty(filename):
            # Future: path lookup by keyword
            # filename = core.locate_path('dataset', filename)
            dataset_content = file.json_to_dict(filename)
            self.update_dataset(dataset_content)
        return ''

    def tag_usetemplate(self, action: mtag.Action):
        template = ''
        filename = self.resolve_token(action.get_argument()[0])
        source_token = action.get_argument()[0]
        if source_token.is_literal():
            filename = source_token.get_value()
        elif source_token.is_name():
            filename = self.get_data('str', source_token.get_value())
        if filename:
            # Future: path lookup by keyword
            # filename = core.locate_path('dataset', filename)
            template_segment = file.file_to_str(filename)
            return self.new_context(template_segment, source='usetemplate')
        return template

    def run(self, prepped_template: list, dataset: dict):
        self.update_dataset(dataset)
        for action in prepped_template:
            self.processed_template.append(self.process_action(action))
        return ''.join(self.processed_template)



#!/usr/bin/python3
from ..generics import file
from . import page as mpage

"""
Site Assembler: Job
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


class Job:

    def __init__(self, config: str):
        self.config = file.json_to_dict(config)
        # self.output_job = self.config.get('sa_output_job')
        self.sites = self.config.pop('sa_sites')

    def run(self):
        for site in self.sites:
            if isinstance(site, str):
                site = file.json_to_dict(site)
            site_config = {**self.config, **site}
            pages = site_config.pop('sa_pages')
            for page in pages:
                if isinstance(page, str):
                    page = file.json_to_dict(page)
                page_config = {**site_config, **page}
                assemble_page = mpage.Page(page_config)
                assemble_page.process()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="djist",
    version="0.1.0",
    author="liorelse",
    author_email="buggyfirmware@protonmail.com",
    description="Django-inspired static templates",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/llelse/djist",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha"
    ],
    python_requires='>=3.6',
)


#!/usr/bin/env python3
from ..generics import core


"""
Site Assembler: Filter
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


def add_filter(value: str, argument: str) -> str:    
    try:
        filtered_value = str(int(value) + int(argument))
    except ValueError:
        filtered_value = value + argument
    except:
        filtered_value = value
    return filtered_value


def addslashes_filter(value: str, argument: str) -> str:
    filtered_value = value.replace("'", "\\'").replace('"', '\\"')
    return filtered_value


def capfirst_filter(value: str, argument: str) -> str:
    filtered_value = value[0].capitalize() + value[1:]
    return filtered_value


def capitalize_filter(value: str, argument: str) -> str:
    filtered_value = value.capitalize()
    return filtered_value


def center_filter(value: str, argument: str) -> str:
    argument_list = argument.split(';', 1)
    try:
        argument_width = int(argument_list.pop(0))
    except ValueError:
        argument_width = 0
    if core.not_empty(argument_list):
        argument_fillchar = argument_list.pop(0)[0]
    else:
        argument_fillchar = ' '
    filtered_value = value.center(argument_width, argument_fillchar)
    return filtered_value


def cut_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def date_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def default_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def default_if_none_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def dictsort_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def dictsortreversed_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def divisibleby_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def escape_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def escapejs_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def filesizeformat_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def first_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def floatformat_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def force_escape_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def get_digit_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def iriencode_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def join_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def json_script_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def last_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def length_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def length_is_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def linebreaks_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def linebreaksbr_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def linenumbers_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def ljust_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def lower_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value.lower()


def make_list_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def phone2numeric_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def pluralize_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def post_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value + argument


def pprint_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def pre_filter(value: str, argument: str) -> str:
    return argument + value


def random_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def rjust_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def safe_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def safeseq_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def slice_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def slugify_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def stringformat_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def striptags_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def time_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def timesince_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def timeuntil_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def title_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def truncatechars_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def truncatechars_html_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def truncatewords_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def truncatewords_html_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def unordered_list_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def upper_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def urlencode_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def urlize_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def urlizetrunc_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def wordcount_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def wordwrap_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


def yesno_filter(value: str, argument: str) -> str:
    filtered_value = value
    return filtered_value


filterselect = {
    'add': add_filter,
    'addslashes': addslashes_filter,
    'capfirst': capfirst_filter,
    'capitalize': capitalize_filter,
    'center': center_filter,
    'cut': cut_filter,
    'date': date_filter,
    'default': default_filter,
    'default_if_none': default_if_none_filter,
    'dictsort': dictsort_filter,
    'dictsortreversed': dictsortreversed_filter,
    'divisibleby': divisibleby_filter,
    'escape': escape_filter,
    'escapejs': escapejs_filter,
    'filesizeformat': filesizeformat_filter,
    'first': first_filter,
    'floatformat': floatformat_filter,
    'force_escape': force_escape_filter,
    'get_digit': get_digit_filter,
    'iriencode': iriencode_filter,
    'join': join_filter,
    'json_script': json_script_filter,
    'last': last_filter,
    'length': length_filter,
    'length_is': length_is_filter,
    'linebreaks': linebreaks_filter,
    'linebreaksbr': linebreaksbr_filter,
    'linenumbers': linenumbers_filter,
    'ljust': ljust_filter,
    'lower': lower_filter,
    'make_list': make_list_filter,
    'phone2numeric': phone2numeric_filter,
    'pluralize': pluralize_filter,
    'post': post_filter,
    'pprint': pprint_filter,
    'pre': pre_filter,
    'random': random_filter,
    'rjust': rjust_filter,
    'safe': safe_filter,
    'safeseq': safeseq_filter,
    'slice': slice_filter,
    'slugify': slugify_filter,
    'stringformat': stringformat_filter,
    'striptags': striptags_filter,
    'time': time_filter,
    'timesince': timesince_filter,
    'timeuntil': timeuntil_filter,
    'title': title_filter,
    'truncatechars': truncatechars_filter,
    'truncatechars_html': truncatechars_html_filter,
    'truncatewords': truncatewords_filter,
    'truncatewords_html': truncatewords_html_filter,
    'unordered_list': unordered_list_filter,
    'upper': upper_filter,
    'urlencode': urlencode_filter,
    'urlize': urlize_filter,
    'urlizetrunc': urlizetrunc_filter,
    'wordcount': wordcount_filter,
    'wordwrap': wordwrap_filter,
    'yesno': yesno_filter,
}


def main():
    """ Main entry point of the app """


if __name__ == "__main__":
    """ This is executed when run from the command line """
    main()



#!/usr/bin/python3
import os
import json
from . import core
# from pyparsing import ParseBaseException, StringStart

"""
Site Assembler: Generic file operations
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


def evaluate(exp, vars):
    pass
    # parser = expression.Expression_Parser(variables=vars)
    # return parser.parse(exp)


def path_exists(path: str):
    if core.not_none_or_empty(path):
        return os.path.exists(path)


def path_join(*dirs: str):
    return os.path.join('', *dirs).replace('\\', '/')


def path_create(path: str):
    if core.not_none_or_empty(path):
        if not path_exists(path):
            path_list = path.split('/')
            if path_list[-1] == '':
                del path_list[-1]
            current = ''
            for directory in path_list:
                current = path_join(current, directory)
                if not path_exists(current):
                    try:
                        os.mkdir(current)
                    except OSError as error:
                        print(error)  # Future : Log error


def write_file(data, filename: str, path: str = 'output', outformat: str = ''):
    if core.not_none_or_empty(data) and core.not_none_or_empty(filename):
        path_create(path)
        full_path = path_join(path, filename)
        try:
            with open(full_path, 'w') as file:
                if outformat == 'json':
                    json.dump(data, file, indent=4)
                else:
                    if isinstance(data, list):
                        file.writelines(data)
                    elif isinstance(data, str):
                        file.write(data)
        except OSError:
            pass


def file_to_list(filename):
    return file_to_str(filename).splitlines()


def file_to_str(filename):
    if os.path.isfile(filename):
        with open(filename, 'r') as file:
            template_str = file.read()
        file.close()
    else:
        template_str = ''
    return template_str


def json_to_dict(filename):
    if os.path.isfile(filename):
        with open(filename) as json_file:
            json_dict = json.load(json_file)
        json_file.close()
    else:
        json_dict = {}
    return json_dict


def data_filename(key):
    return str('data.' + key + '.txt').lower()


def template_filename(key):
    return str('template.' + key + '.txt').lower()



#!/usr/bin/python3


"""
Site Assembler: Generic functions
"""

__author__ = "llelse"
__version__ = "0.1.0"
__license__ = "GPLv3"


def evaluate(exp, vars):
    pass
    # parser = expression.Expression_Parser(variables=vars)
    # return parser.parse(exp)


def is_empty(obj):
    return len(obj) == 0


def not_empty(obj):
    return not is_empty(obj)


def is_none_or_empty(obj):
    return is_empty(obj) and obj is not None


def not_none_or_empty(obj):
    return not is_none_or_empty(obj)


def web_safe_subs():
    return {
        '"': '&#34;',
        "'": '&#39;',
        # '--': '<br />'
    }


def substitute(subs: dict, text: str):
    for k, v in subs.items():
        text = text.replace(k, v)
    return text


def web_safe_string(dirty_string: str):
    safe_string = substitute(web_safe_subs(), dirty_string)
    return safe_string


def web_safe_list(dirty_list: list):
    safe_list = []
    for x in dirty_list:
        safe_list.append(web_safe_string(x))
    return safe_list
